from tkinter import *
import math
#
# ■相対描画のためのRelativeDrawクラス定義
#　  
#   変数名 = RelativeDraw(canvas, startX, startY,scaleX, scaleY, lineWidth,color)
#
#       canvas          : キャンバス
#       startX, startY  : 表示開始位置
#       scaleX, scaleY  : 表示スケール(上下または左右を逆転するときは負の値で指定)
#       lineWidth       : 描画する直線の太さ
#       color           : カラーを文字列で指定(例 : 'red', '#00FFFF')
#
class RelativeDraw:
    #■ コンストラクタ
    def __init__(self, canvas,startX, startY,scaleX, scaleY, lineWidth,color):
        self.canvas=canvas
        self.X0=startX
        self.Y0=startY
        self.SX=scaleX
        self.SY=scaleY
        self.lineWidth=lineWidth
        self.color=color
        self.goto(0,0)
        
    #■ XY座標設定
    def goto(self,x,y):
        self.x=x
        self.y=y
        
    #■ スクリーン座標への変換
    def screenX(self, X):
        return self.X0 + self.SX *  X
    def screenY(self, Y):
        return self.Y0 + self.SY *  Y

class TurtleGraph(RelativeDraw):
    #■ コンストラクタ
    def __init__(self, canvas,startX, startY,scaleX, scaleY, lineWidth,color):
        RelativeDraw.__init__(self,canvas,startX, startY,scaleX, scaleY,
                              lineWidth,color)
        self.setAng(0)
    #■ 角度設定
    def setAng(self,th):
        self.th=th

    #■ 向きの変更(左回転は負の値で指定)
    def turn(self,th):
        self.th=self.th+th

    #■ 前に進む(後退は負の値で指定)
    def forward(self,L):
        TH=self.th*math.pi/180
        X=self.x+L*math.cos(TH)
        Y=self.y+L*math.sin(TH)
        self.canvas.create_line(self.screenX(self.x), self.screenY(self.y),
                                self.screenX(X)     , self.screenY(Y)     ,
                                width = self.lineWidth, fill = self.color)
        self.x=X
        self.y=Y

        
#■相対位置まで曲線を描くクラス
class RelativeCurve(RelativeDraw):
    def draw(self,X,Y):
        NX=self.x+X
        NY=self.y+Y
        self.canvas.create_line(self.screenX(self.x), self.screenY(self.y),
                                self.screenX(NX)     , self.screenY(NY)     ,
                                width = self.lineWidth, fill = self.color)
        self.x=NX
        self.y=NY
        
#■ドラゴン曲線のクラス定義
#  (主処理以外はRelativeCurve, RelativeDrawクラスからの継承)
class DragonCurve(RelativeCurve):
    def curve(self,X,Y,S,ML):
        if X*X+Y*Y<ML*ML:
            self.draw(X,Y)
        else:
            self.curve((X-S*Y)/2, (Y+S*X)/2, 1, ML)
            self.curve((X+S*Y)/2, (Y-S*X)/2,-1, ML)

#■ tkinter，Canvas共通処理
def initTk(Title):
    tk=Tk()
    tk.title(Title)
    return tk
def initCanvas(W,H):
    canvas=Canvas(tk,width=W,height=H)
    canvas.pack()
    return canvas

#■　メイン処理
tk=initTk("Dragon Curve used tkinter")
canvas=initCanvas(600,300)
dragon=DragonCurve(canvas, 150, 180, 1, -1, 1, 'blue')
dragon.curve(200,0,1,2)
tk.mainloop()

            
        
